       IDENTIFICATION DIVISION.
       PROGRAM-ID.    CUUPDC01.
       AUTHOR.        SIMOTIME TECHNOLOGIES.
      *****************************************************************
      * Copyright (C) 1987-2016 SimoTime Technologies.                *
      *                                                               *
      * All rights reserved.  Unpublished, all rights reserved under  *
      * copyright law and international treaty.  Use of a copyright   *
      * notice is precautionary only and does not imply publication   *
      * or disclosure.                                                *
      *                                                               *
      * Permission to use, copy, modify and distribute this software  *
      * for any commercial purpose requires a fee to be paid to       *
      * SimoTime Technologies. Once the fee is received by SimoTime   *
      * the latest version of the software will be delivered and a    *
      * license will be granted for use within an enterprise,         *
      * provided the SimoTime copyright notice appear on all copies   *
      * of the software. The SimoTime name or Logo may not be used    *
      * in any advertising or publicity pertaining to the use of the  *
      * software without the written permission of SimoTime           *
      * Technologies.                                                 *
      *                                                               *
      * Permission to use, copy and modify this software for any      *
      * non-commercial purpose and without fee is hereby granted,     *
      * provided the SimoTime copyright notice appear on all copies   *
      * of the software. The SimoTime name or Logo may not be used in *
      * any advertising or publicity pertaining to the use of the     *
      * software without the written permission of SimoTime           *
      * Technologies.                                                 *
      *                                                               *
      * SimoTime Technologies makes no warranty or representations    *
      * about the suitability of the software for any purpose. It is  *
      * provided "AS IS" without any expressed or implied warranty,   *
      * including the implied warranties of merchantability, fitness  *
      * for a particular purpose and non-infringement. SimoTime       *
      * Technologies shall not be liable for any direct, indirect,    *
      * special or consequential damages resulting from the loss of   *
      * use, data or projects, whether in an action of contract or    *
      * tort, arising out of or in connection with the use or         *
      * performance of this software                                  *
      *                                                               *
      * SimoTime Technologies                                         *
      * 15 Carnoustie Drive                                           *
      * Novato, CA 94949-5849                                         *
      * 415.883.6565                                                  *
      *                                                               *
      * RESTRICTED RIGHTS LEGEND                                      *
      * Use, duplication, or disclosure by the Government is subject  *
      * to restrictions as set forth in subparagraph (c)(1)(ii) of    *
      * the Rights in Technical Data and Computer Software clause at  *
      * DFARS 52.227-7013 or subparagraphs (c)(1) and (2) of          *
      * Commercial  Computer Software - Restricted Rights  at 48      *
      * CFR 52.227-19, as applicable.  Contact SimoTime Technologies, *
      * 15 Carnoustie Drive, Novato, CA 94949-5849.                   *
      *                                                               *
      *****************************************************************
      *      This program is provided by SimoTime Technologies        *
      *        Our e-mail address is: helpdesk@simotime.com           *
      *     Also, visit our Web Site at http://www.simotime.com       *
      *                                                               *
      *****************************************************************
      *
      *****************************************************************
      * Source Member: CUUPDC01.CBL
      * Copy Files:    QQSMREC1.CPY
      *****************************************************************
      *
      * CUUPDC01 - Execute CUUPDC01 to read a QSAM file and  update
      * a VSAM Keyed Sequential Data Set (KSDS).
      *
      * EXECUTION or CALLING PROTOCOL
      * -----------------------------
      * Use standard JCL to EXECUTE or ANIMATE.
      *
      * DESCRIPTION
      * -----------
      * This single COBOL program will read an 80-byte, EBCDIC, QSAM
      * file an populate or update an empty or existing KSDS, VSAM
      * data set.
      *
      *                      ************
      *                      * KSDUPD *
      *                      ********jcl*
      *                           *
      *                           *
      *     ************     ************     ************
      *     * CUSTSQ01 *-----* CUUPDC01 *-----* CUSTMAST *
      *     *******qsam*     ********cbl*     *******vsam*
      *                           *
      *                           *
      *                      ************
      *                      *   EOJ    *
      *                      ************
      *
      *****************************************************************
      * This program will read the input file and update the master
      * file (SYSMLR2). The format of the input file (SYSMLR1) is
      * as follows.
      *
      * Position Length Description...
      *  ------   ----  -----------------------------------------------
      *    01      03   Record Idendifier
      *                 /N1 Name change or correction
      *                 /A1 Address change or correction (street)
      *                 /CT City
      *                 /ST State
      *                 /PC Postal Code
      *    04      12   The Key Field
      *    16      nn   Length based on Record Identifier
      *
      *****************************************************************
      *
      * MAINTENANCE
      * -----------
      * 1997/02/27 Simmons, Created program.
      * 1997/02/27 Simmons, No changes to date.
      *
      *****************************************************************
      *
       ENVIRONMENT DIVISION.
       INPUT-OUTPUT   SECTION.
       FILE-CONTROL.

      *****************************************************************
           SELECT CUSTSQ01-FILE
               ASSIGN       to CUSTSQ01
               ORGANIZATION is SEQUENTIAL
               ACCESS MODE  is SEQUENTIAL
               FILE STATUS  is CUSTSQ01-STATUS.
           SELECT CUSTMAST-FILE
               ASSIGN       to CUSTMAST
               ORGANIZATION is indexed
               ACCESS MODE  is RANDOM
               RECORD KEY   is CUST-NUMBER
               FILE STATUS  is CUSTMAST-STATUS.

      *****************************************************************
       DATA DIVISION.
       FILE SECTION.

      *****************************************************************
       FD   CUSTSQ01-FILE
            RECORD CONTAINS 80 CHARACTERS.
       COPY QSMCPYB2.

       FD  CUSTMAST-FILE.
       COPY CUSTCB01.

       WORKING-STORAGE SECTION.
      *****************************************************************
      *    Data-structure for Title and Copyright...
      *****************************************************************
       01  SIM-TITLE.
           05  T1 pic X(11) value '* CUUPDC01 '.
           05  T2 pic X(34) value ' Read CUSTSQ01, Update CUSTMAST '.
           05  T3 pic X(10) value ' v07.02.21'.
           05  T4 pic X(24) value ' http://www.simotime.com'.
       01  SIM-COPYRIGHT.
           05  C1 pic X(11) value '* CUUPDC01 '.
           05  C2 pic X(20) value 'Copyright 1987-2016 '.
           05  C3 pic X(28) value '   SimoTime Technologies    '.
           05  C4 pic X(20) value ' All Rights Reserved'.

       01  SIM-THANKS-01.
           05  C1 pic X(11) value '* CUUPDC01 '.
           05  C2 pic X(32) value 'Thank you for using this program'.
           05  C3 pic X(32) value ' provided from SimoTime Technolo'.
           05  C4 pic X(04) value 'gies'.

       01  SIM-THANKS-02.
           05  C1 pic X(11) value '* CUUPDC01 '.
           05  C2 pic X(32) value 'Please send all inquires or sugg'.
           05  C3 pic X(32) value 'estions to the helpdesk@simotime'.
           05  C4 pic X(04) value '.com'.

       01  CUSTMAST-STATUS.
           05  CUSTMAST-STAT1      pic X.
           05  CUSTMAST-STAT2      pic X.

       01  CUSTSQ01-STATUS.
           05  CUSTSQ01-STAT1      pic X.
           05  CUSTSQ01-STAT2      pic X.

       01  IO-STATUS.
           05  IO-STAT1            pic X.
           05  IO-STAT2            pic X.
       01  TWO-BYTES.
           05  TWO-BYTES-LEFT      pic X.
           05  TWO-BYTES-RIGHT     pic X.
       01  TWO-BYTES-BINARY        redefines TWO-BYTES pic 9(4) comp.

       01  END-OF-FILE             pic X(3)    value 'NO '.

      *****************************************************************
      *    Buffer used for posting messages to the console.
      *****************************************************************
       01  MESSAGE-BUFFER.
           05  MESSAGE-HEADER      pic X(11)   value '* CUUPDC01 '.
           05  MESSAGE-TEXT.
               10  MESSAGE-TEXT-1  pic X(68)   value SPACES.
               10  MESSAGE-TEXT-2  pic X(188)  value SPACES.

       01  CUSTSQ01-FLAG           pic X       value 'N'.

       01  WORK-07.
           05  WORK-07-NUMB        pic 9(7)    value 0.
       01  WX-7                    pic 9(2)    value 0.

       01  APPL-RESULT             pic S9(9)   comp.
           88  APPL-AOK            value 0.
           88  APPL-EOF            value 16.

      *****************************************************************
       PROCEDURE DIVISION.
           perform Z-POST-COPYRIGHT.

           perform CUSTSQ01-OPEN.
           perform CUSTMAST-OPEN.

           perform until END-OF-FILE = 'YES'
               if  END-OF-FILE = 'NO '
                   perform CUSTSQ01-GET
                   if  END-OF-FILE = 'NO '
                       move QSAM-B2-RECORD to MESSAGE-TEXT
                       perform Z-DISPLAY-MESSAGE-TEXT
                       move QSAM-B2-KEY to CUST-NUMBER
                       perform CUSTMAST-GET
                       perform EVALUATE-FIELD-TYPE
                       if  CUSTSQ01-FLAG = 'Y'
                           if  APPL-AOK
                               perform MOVE-QSAM-FIELD-TO-KSDS-RECORD
                               perform CUSTMAST-REWRITE
                           else
                               perform INITIALIZE-CUSTOMER-RECORD
                               move QSAM-B2-KEY to CUST-NUMBER
                               perform MOVE-QSAM-FIELD-TO-KSDS-RECORD
                               perform CUSTMAST-WRITE
                           end-if
                       end-if
                   end-if
               end-if
           end-perform.

           perform CUSTMAST-CLOSE.
           perform CUSTSQ01-CLOSE.

           display 'CUUPDC01 CUSTMAST-HAS-BEEN-UPDATED'
           display 'CUUPDC01 NORMAL-END-OF-JOB...'

           perform Z-THANK-YOU.

           GOBACK.

      *****************************************************************
      * The following routines are in alphabetic sequence.            *
      *****************************************************************
       INITIALIZE-CUSTOMER-RECORD.
           move all SPACES to CUST-RECORD
           initialize CUST-RECORD
           exit.

      *****************************************************************
       EVALUATE-FIELD-TYPE.
           if  QSAM-B2-ID = '/NM'
           or  QSAM-B2-ID = '/NL'
           or  QSAM-B2-ID = '/NF'
           or  QSAM-B2-ID = '/A1'
           or  QSAM-B2-ID = '/A2'
           or  QSAM-B2-ID = '/CT'
           or  QSAM-B2-ID = '/ST'
           or  QSAM-B2-ID = '/PC'
           or  QSAM-B2-ID = '/CL'
           or  QSAM-B2-ID = '/TH'
           or  QSAM-B2-ID = '/TO'
           or  QSAM-B2-ID = '/TM'
               move 'Y' to CUSTSQ01-FLAG
           else
               move 'N' to CUSTSQ01-FLAG
           end-if
           exit.
      *****************************************************************
       MOVE-QSAM-FIELD-TO-KSDS-RECORD.
           move 'Y' to CUSTSQ01-FLAG
           evaluate QSAM-B2-ID
               when '/NL' move QSAM-B2-DATA to CUST-LAST-NAME
               when '/NF' move QSAM-B2-DATA to CUST-FIRST-NAME
               when '/NM' move QSAM-B2-DATA to CUST-MID-NAME
               when '/A1' move QSAM-B2-DATA to CUST-ADDRESS-1
               when '/A2' move QSAM-B2-DATA to CUST-ADDRESS-2
               when '/CT' move QSAM-B2-DATA to CUST-CITY
               when '/ST' move QSAM-B2-DATA to CUST-STATE
               when '/PC' move QSAM-B2-DATA to CUST-POSTAL-CODE
               when '/CL' perform MOVE-CUST-CREDIT-LIMIT
               when '/TH' move QSAM-B2-DATA to CUST-PHONE-HOME
               when '/TO' move QSAM-B2-DATA to CUST-PHONE-WORK
               when '/TM' move QSAM-B2-DATA to CUST-PHONE-CELL
               when other move 'N' to CUSTSQ01-FLAG
           end-evaluate

           exit.

      *****************************************************************
       MOVE-CUST-CREDIT-LIMIT.
           move QSAM-B2-DATA to WORK-07
           perform until WORK-07(7:1) not = SPACE
             perform varying WX-7 from 6 by -1 until WX-7 = 0
               move WORK-07(WX-7:1) to WORK-07(WX-7 + 1:1)
             end-perform
      *      move WORK-07(6:1) to WORK-07(7:1)
      *      move WORK-07(5:1) to WORK-07(6:1)
      *      move WORK-07(4:1) to WORK-07(5:1)
      *      move WORK-07(3:1) to WORK-07(4:1)
      *      move WORK-07(2:1) to WORK-07(3:1)
      *      move WORK-07(1:1) to WORK-07(2:1)
             move ZERO         to WORK-07(1:1)
           end-perform
           if  WORK-07-NUMB is NUMERIC
               add WORK-07-NUMB to ZERO giving CUST-CREDIT-LIMIT
           else
               move 'Credit Limit Update Error...' to MESSAGE-TEXT
               perform Z-DISPLAY-MESSAGE-TEXT
           end-if
           exit.

      *****************************************************************
      * Routines to do a sequential read of the QSAM file.            *
      *****************************************************************
       CUSTSQ01-GET.
           read CUSTSQ01-FILE
           if  CUSTSQ01-STATUS = '00'
               subtract APPL-RESULT from APPL-RESULT
           else
               if  CUSTSQ01-STATUS = '10'
                   add 16 to ZERO giving APPL-RESULT
               else
                   add 12 to ZERO giving APPL-RESULT
               end-if
           end-if
           if  APPL-AOK
               CONTINUE
           else
               if  APPL-EOF
                   move 'YES' to END-OF-FILE
               else
                   move 'CUUPDC01 CUSTSQ01-FAILURE-GET...'
                     to   MESSAGE-TEXT
                   move CUSTSQ01-STATUS to IO-STATUS
                   perform Z-DISPLAY-MESSAGE-TEXT
                   perform Z-DISPLAY-IO-STATUS
                   perform Z-ABEND-PROGRAM
               end-if
           end-if
           exit.
      *---------------------------------------------------------------*
       CUSTSQ01-OPEN.
           add 8 to ZERO giving APPL-RESULT.
           open input CUSTSQ01-FILE
           if  CUSTSQ01-STATUS = '00'
               subtract APPL-RESULT from APPL-RESULT
           else
               add 12 to ZERO giving APPL-RESULT
           end-if
           if  APPL-AOK
               CONTINUE
           else
               move 'CUUPDC01 CUSTSQ01-FAILURE-OPEN...'
                 to MESSAGE-TEXT
               move CUSTSQ01-STATUS to IO-STATUS
               perform Z-DISPLAY-MESSAGE-TEXT
               perform Z-DISPLAY-IO-STATUS
               perform Z-ABEND-PROGRAM
           end-if
           exit.
      *---------------------------------------------------------------*
       CUSTSQ01-CLOSE.
           add 8 to ZERO giving APPL-RESULT.
           close CUSTSQ01-FILE
           if  CUSTSQ01-STATUS = '00'
               subtract APPL-RESULT from APPL-RESULT
           else
               add 12 to ZERO giving APPL-RESULT
           end-if

           if  APPL-AOK
               CONTINUE
           else
               move 'CUUPDC01, CUSTSQ01, FAILURE, CLOSE...'
                 to MESSAGE-TEXT
               move CUSTSQ01-STATUS to IO-STATUS
               perform Z-DISPLAY-MESSAGE-TEXT
               perform Z-DISPLAY-IO-STATUS
               perform Z-ABEND-PROGRAM
           end-if
           exit.

      *****************************************************************
      * Routines to do a read by KEY of the KSDS, VSAM Data Set. If   *
      * the read is successful then the record may be updated else a  *
      * new record may be added.                                      *
      *****************************************************************
       CUSTMAST-GET.
           read CUSTMAST-FILE
           if  CUSTMAST-STATUS = '00'
               subtract APPL-RESULT from APPL-RESULT
           else
               if  CUSTMAST-STATUS = '10'
                   add 16 to ZERO giving APPL-RESULT
               else
                   add 12 to ZERO giving APPL-RESULT
               end-if
           end-if
           if  APPL-AOK
               CONTINUE
           else
               if  APPL-EOF
                   move 'YES' to END-OF-FILE
               end-if
           end-if
           exit.
      *---------------------------------------------------------------*
       CUSTMAST-WRITE.
           write CUST-RECORD
           if  CUSTMAST-STATUS = '00'
               subtract APPL-RESULT from APPL-RESULT
           else
               if  CUSTMAST-STATUS = '10'
                   add 16 to ZERO giving APPL-RESULT
               else
                   add 12 to ZERO giving APPL-RESULT
               end-if
           end-if
           if  APPL-AOK
               CONTINUE
           else
               if  APPL-EOF
                   move 'YES' to END-OF-FILE
               else
                   move 'CUUPDC01 CUSTMAST-FAILURE-WRITE...'
                     to   MESSAGE-TEXT
                   move CUSTMAST-STATUS to IO-STATUS
                   perform Z-DISPLAY-MESSAGE-TEXT
                   perform Z-DISPLAY-IO-STATUS
                   perform Z-ABEND-PROGRAM
               end-if
           end-if
           exit.
      *---------------------------------------------------------------*
       CUSTMAST-REWRITE.
           REWRITE CUST-RECORD
           if  CUSTMAST-STATUS = '00'
               subtract APPL-RESULT from APPL-RESULT
           else
               if  CUSTMAST-STATUS = '10'
                   add 16 to ZERO giving APPL-RESULT
               else
                   add 12 to ZERO giving APPL-RESULT
               end-if
           end-if
           if  APPL-AOK
               CONTINUE
           else
               if  APPL-EOF
                   move 'YES' to END-OF-FILE
               else
                   move 'CUUPDC01 CUSTMAST-FAILURE-REWRITE...'
                     to   MESSAGE-TEXT
                   move CUSTMAST-STATUS to IO-STATUS
                   perform Z-DISPLAY-MESSAGE-TEXT
                   perform Z-DISPLAY-IO-STATUS
                   perform Z-ABEND-PROGRAM
               end-if
           end-if
           exit.
      *---------------------------------------------------------------*
       CUSTMAST-OPEN.
           add 8 to ZERO giving APPL-RESULT.
           open I-O CUSTMAST-FILE
           if  CUSTMAST-STATUS = '35'
               move 'CUUPDC01 CUSTMAST-FAILURE-OPEN-IO...'
                 to MESSAGE-TEXT
               perform Z-DISPLAY-MESSAGE-TEXT
               move CUSTMAST-STATUS to IO-STATUS
               perform Z-DISPLAY-IO-STATUS
               move 'CUUPDC01 CUSTMAST-ATTEMPT-OPEN-OUTPUT...'
                 to MESSAGE-TEXT
               perform Z-DISPLAY-MESSAGE-TEXT
               open output CUSTMAST-FILE
           end-if
           if  CUSTMAST-STATUS = '00'
               subtract APPL-RESULT from APPL-RESULT
           else
               add 12 to ZERO giving APPL-RESULT
           end-if
           if  APPL-AOK
               CONTINUE
           else
               move 'CUUPDC01 CUSTMAST-FAILURE-OPEN...'
                 to MESSAGE-TEXT
               move CUSTMAST-STATUS to IO-STATUS
               perform Z-DISPLAY-MESSAGE-TEXT
               perform Z-DISPLAY-IO-STATUS
               perform Z-ABEND-PROGRAM
           end-if
           exit.
      *---------------------------------------------------------------*
       CUSTMAST-CLOSE.
           add 8 to ZERO giving APPL-RESULT.
           close CUSTMAST-FILE
           if  CUSTMAST-STATUS = '00'
               subtract APPL-RESULT from APPL-RESULT
           else
               add 12 to ZERO giving APPL-RESULT
           end-if
           if  APPL-AOK
               CONTINUE
           else
               move 'CUUPDC01 CUSTMAST-FAILURE-CLOSE...'
                 to   MESSAGE-TEXT
               move CUSTMAST-STATUS to IO-STATUS
               perform Z-DISPLAY-MESSAGE-TEXT
               perform Z-DISPLAY-IO-STATUS
               perform Z-ABEND-PROGRAM
           end-if
           exit.

      *****************************************************************
      * The following Z-Routines perform administrative tasks         *
      * for this program.                                             *
      *****************************************************************
      *
      *****************************************************************
      * ABEND the program, post a message to the console and issue    *
      * a STOP RUN.                                                   *
      *****************************************************************
       Z-ABEND-PROGRAM.
           if  MESSAGE-TEXT not = SPACES
               perform Z-DISPLAY-MESSAGE-TEXT
           end-if
           move 'CUUPDC01 PROGRAM-IS-ABENDING...'  to MESSAGE-TEXT
           perform Z-DISPLAY-MESSAGE-TEXT
           add 12 to ZERO giving RETURN-CODE
           STOP RUN.

      *****************************************************************
      * Display the file status bytes. This routine will display as   *
      * two digits if the full two byte file status is numeric. If    *
      * second byte is non-numeric then it will be treated as a       *
      * binary number.                                                *
      *****************************************************************
       Z-DISPLAY-IO-STATUS.
           if  IO-STATUS not NUMERIC
           or  IO-STAT1 = '9'
               subtract TWO-BYTES-BINARY from TWO-BYTES-BINARY
               move IO-STAT2 to TWO-BYTES-RIGHT
               display '* CUUPDC01 FILE-STATUS-' IO-STAT1 '/'
                       TWO-BYTES-BINARY
           else
               display '* CUUPDC01 FILE-STATUS-' IO-STATUS
           end-if
           exit.

      *****************************************************************
       Z-DISPLAY-MESSAGE-TEXT.
           if MESSAGE-TEXT-2 = SPACES
               display MESSAGE-BUFFER(1:79)
           else
               display MESSAGE-BUFFER
           end-if
           move all SPACES to MESSAGE-TEXT
           exit.

      *****************************************************************
       Z-POST-COPYRIGHT.
           display SIM-TITLE
           display SIM-COPYRIGHT
           exit.

      *****************************************************************
       Z-THANK-YOU.
           display SIM-THANKS-01
           display SIM-THANKS-02
           exit.
      *****************************************************************
      *      This example is provided by SimoTime Technologies        *
      *        Our e-mail address is: helpdesk@simotime.com           *
      *     Also, visit our Web Site at http://www.simotime.com       *
      *****************************************************************
